@@ -4,7 +4,14 @@
 use core::mem;
 
 use network_types::{
-    arp::ArpHdr, eth::EthHdr, icmp::Icmp, ip::IpHdr, sctp::SctpHdr, tcp::TcpHdr, udp::UdpHdr,
+    arp::ArpHdr,
+    eth::EthHdr,
+    icmp::Icmp,
+    igmp::{IGMPv1Hdr, IGMPv2Hdr, IGMPv3MembershipQueryHdr, IGMPv3MembershipReportHdr},
+    ip::IpHdr,
+    sctp::SctpHdr,
+    tcp::TcpHdr,
+    udp::UdpHdr,
 };
 
 pub mod protocols;
@@ -65,4 +72,20 @@ pub enum ProtoHdr {
     Udp(UdpHdr),
     Sctp(SctpHdr),
     Icmp(Icmp),
+    Igmp(IgmpHdr),
+}
+
+#[derive(Copy, Clone)]
+#[repr(C)]
+pub enum IgmpHdr {
+    V1(IGMPv1Hdr),
+    V2(IGMPv2Hdr),
+    V3(IGMPv3Hdr),
+}
+
+#[derive(Copy, Clone)]
+#[repr(C)]
+pub enum IGMPv3Hdr {
+    Query(IGMPv3MembershipQueryHdr),
+    Report(IGMPv3MembershipReportHdr),
 }
