@@ -1,4 +1,4 @@
-use std::sync::{Arc, Mutex, RwLock};
+use std::sync::{Arc, Mutex};
 
 use crossterm::event::{KeyCode, KeyEvent};
 use ratatui::{
@@ -23,40 +23,41 @@ use crate::{
         eth_frame::EthFrameHeader,
         network::{IpPacket, ip::IpProto},
     },
+    packet_store::PacketStore,
 };
 
 #[derive(Debug)]
 pub struct Inspection {
-    pub packets: Arc<RwLock<Vec<AppPacket>>>,
+    pub packets: PacketStore,
     pub state: TableState,
     pub fuzzy: Arc<Mutex<Fuzzy>>,
-    pub manuall_scroll: bool,
+    pub manual_scroll: bool,
     pub packet_end_index: usize,
     pub packet_window_size: usize,
     pub packet_index: Option<usize>,
+    pub packets_display_buffer: Vec<AppPacket>,
 }
 
 impl Inspection {
-    pub fn new(packets: Arc<RwLock<Vec<AppPacket>>>) -> Self {
+    pub fn new(packets: PacketStore) -> Self {
         Self {
             packets: packets.clone(),
             state: TableState::default(),
             fuzzy: Fuzzy::new(packets.clone()),
-            manuall_scroll: false,
+            manual_scroll: false,
             packet_end_index: 0,
             packet_window_size: 0,
             packet_index: None,
+            packets_display_buffer: Vec::with_capacity(128),
         }
     }
 
     pub fn can_show_popup(&mut self) -> bool {
-        let packets = self.packets.read().unwrap();
         let fuzzy = self.fuzzy.lock().unwrap();
-
         if fuzzy.is_enabled() {
             !fuzzy.packets.is_empty()
         } else {
-            !packets.is_empty()
+            !self.packets.is_empty()
         }
     }
 
@@ -73,8 +74,8 @@ impl Inspection {
                 KeyCode::Esc => {
                     if !fuzzy.is_paused() {
                         fuzzy.pause();
-                    } else if self.manuall_scroll {
-                        self.manuall_scroll = false;
+                    } else if self.manual_scroll {
+                        self.manual_scroll = false;
                     } else {
                         fuzzy.disable();
                     }
@@ -87,8 +88,8 @@ impl Inspection {
                     } else {
                         match key_event.code {
                             KeyCode::Char('j') | KeyCode::Down => {
-                                if !self.manuall_scroll {
-                                    self.manuall_scroll = true;
+                                if !self.manual_scroll {
+                                    self.manual_scroll = true;
                                     fuzzy.packet_end_index = fuzzy.packets.len();
                                 }
                                 fuzzy.scroll_down(self.packet_window_size);
@@ -100,8 +101,8 @@ impl Inspection {
                             }
 
                             KeyCode::Char('k') | KeyCode::Up => {
-                                if !self.manuall_scroll {
-                                    self.manuall_scroll = true;
+                                if !self.manual_scroll {
+                                    self.manual_scroll = true;
                                     fuzzy.packet_end_index = fuzzy.packets.len();
                                 }
                                 fuzzy.scroll_up(self.packet_window_size);
@@ -115,8 +116,8 @@ impl Inspection {
         } else {
             match key_event.code {
                 KeyCode::Esc => {
-                    if self.manuall_scroll {
-                        self.manuall_scroll = false;
+                    if self.manual_scroll {
+                        self.manual_scroll = false;
                     }
                 }
 
@@ -135,15 +136,14 @@ impl Inspection {
                 }
 
                 KeyCode::Char('s') => {
-                    let app_packets = self.packets.read().unwrap();
-                    if app_packets.is_empty() {
+                    if self.packets.is_empty() {
                         Notification::send(
                             "There is no packets".to_string(),
                             NotificationLevel::Info,
                             event_sender,
                         )?;
                     } else {
-                        match export::export(&app_packets) {
+                        match export::export(&self.packets) {
                             Ok(_) => {
                                 Notification::send(
                                     "Packets exported to ~/oryx directory".to_string(),
@@ -169,11 +169,10 @@ impl Inspection {
     }
 
     pub fn scroll_up(&mut self) {
-        let app_packets = self.packets.read().unwrap();
-        if !self.manuall_scroll {
-            self.manuall_scroll = true;
-            // Record the last position. Usefull for selecting the packets to display
-            self.packet_end_index = app_packets.len();
+        if !self.manual_scroll {
+            self.manual_scroll = true;
+            // Record the last position. Useful for selecting the packets to display
+            self.packet_end_index = self.packets.len();
         }
         let i = match self.state.selected() {
             Some(i) => {
@@ -194,19 +193,16 @@ impl Inspection {
     }
 
     pub fn scroll_down(&mut self) {
-        let app_packets = self.packets.read().unwrap();
-
-        if !self.manuall_scroll {
-            self.manuall_scroll = true;
-            self.packet_end_index = app_packets.len();
+        let packets_len = self.packets.len();
+        if !self.manual_scroll {
+            self.manual_scroll = true;
+            self.packet_end_index = packets_len;
         }
         let i = match self.state.selected() {
             Some(i) => {
                 if i < self.packet_window_size - 1 {
                     i + 1
-                } else if i == self.packet_window_size - 1
-                    && app_packets.len() > self.packet_end_index
-                {
+                } else if i == self.packet_window_size - 1 && packets_len > self.packet_end_index {
                     // shift the window by one
                     self.packet_end_index += 1;
                     i + 1
@@ -214,14 +210,13 @@ impl Inspection {
                     i
                 }
             }
-            None => app_packets.len(),
+            None => packets_len,
         };
 
         self.state.select(Some(i));
     }
 
     pub fn render(&mut self, frame: &mut Frame, block: Rect) {
-        let app_packets = self.packets.read().unwrap();
         let mut fuzzy = self.fuzzy.lock().unwrap();
         let fuzzy_packets = fuzzy.clone().packets.clone();
 
@@ -277,7 +272,10 @@ impl Inspection {
             fuzzy.packet_end_index = window_size;
         }
 
-        let packets_to_display = match self.manuall_scroll {
+        let packets_len = self.packets.len();
+        let pdb = &mut self.packets_display_buffer;
+        pdb.clear();
+        match self.manual_scroll {
             true => {
                 if fuzzy.is_enabled() & !fuzzy.filter.value().is_empty() {
                     if fuzzy_packets.len() > window_size {
@@ -286,54 +284,60 @@ impl Inspection {
                                 fuzzy.packet_end_index.saturating_sub(window_size) + selected_index,
                             );
                         }
-                        &fuzzy_packets[fuzzy.packet_end_index.saturating_sub(window_size)
-                            ..fuzzy.packet_end_index]
+                        pdb.extend_from_slice(
+                            &fuzzy_packets[fuzzy.packet_end_index.saturating_sub(window_size)
+                                ..fuzzy.packet_end_index],
+                        );
                     } else {
                         if let Some(selected_index) = fuzzy.scroll_state.selected() {
                             self.packet_index = Some(selected_index);
                         } else {
                             self.packet_index = None;
                         }
-                        &fuzzy_packets
+                        pdb.extend_from_slice(&fuzzy_packets)
                     }
-                } else if app_packets.len() > window_size {
+                } else if packets_len > window_size {
                     if let Some(selected_index) = self.state.selected() {
                         self.packet_index = Some(
                             self.packet_end_index.saturating_sub(window_size) + selected_index,
                         );
                     }
-                    &app_packets
-                        [self.packet_end_index.saturating_sub(window_size)..self.packet_end_index]
+                    self.packets.write_range_into(
+                        self.packet_end_index.saturating_sub(window_size)..self.packet_end_index,
+                        pdb,
+                    );
                 } else {
                     if let Some(selected_index) = self.state.selected() {
                         self.packet_index = Some(selected_index);
                     }
-                    &app_packets
+                    self.packets.write_range_into(0..packets_len, pdb);
                 }
             }
             false => {
                 if fuzzy.is_enabled() & !fuzzy.filter.value().is_empty() {
                     if fuzzy_packets.len() > window_size {
                         self.packet_index = Some(fuzzy_packets.len().saturating_sub(1));
-                        &fuzzy_packets[fuzzy_packets.len().saturating_sub(window_size)..]
+                        pdb.extend_from_slice(
+                            &fuzzy_packets[fuzzy_packets.len().saturating_sub(window_size)..],
+                        )
                     } else {
                         self.packet_index = Some(fuzzy_packets.len().saturating_sub(1));
-                        &fuzzy_packets
+                        pdb.extend_from_slice(&fuzzy_packets);
                     }
-                } else if app_packets.len() > window_size {
-                    self.packet_index = Some(app_packets.len().saturating_sub(1));
-                    &app_packets[app_packets.len().saturating_sub(window_size)..]
+                } else if packets_len > window_size {
+                    let start_index = packets_len.saturating_sub(window_size);
+                    self.packet_index = Some(packets_len.saturating_sub(1));
+                    self.packets.write_range_into(start_index..packets_len, pdb);
                 } else {
-                    self.packet_index = Some(app_packets.len().saturating_sub(1));
-                    &app_packets
+                    self.packet_index = Some(packets_len.saturating_sub(1));
+                    self.packets.write_range_into(0..packets_len, pdb);
                 }
             }
         };
 
         // Style the packets
         let packets: Vec<Row> = if fuzzy.is_enabled() & !fuzzy.filter.value().is_empty() {
-            packets_to_display
-                .iter()
+            pdb.iter()
                 .map(|app_packet| {
                     let pid = match app_packet.pid {
                         Some(pid) => fuzzy::highlight(pattern, pid.to_string()).blue(),
@@ -391,6 +395,16 @@ impl Inspection {
                                     fuzzy::highlight(pattern, "ICMPv4".to_string()).cyan(),
                                     pid,
                                 ]),
+                                IpProto::Igmp(_) => Row::new(vec![
+                                    fuzzy::highlight(pattern, ipv4_packet.src_ip.to_string())
+                                        .blue(),
+                                    Cell::from(Line::from("-").centered()).yellow(),
+                                    fuzzy::highlight(pattern, ipv4_packet.dst_ip.to_string())
+                                        .blue(),
+                                    Cell::from(Line::from("-").centered()).yellow(),
+                                    fuzzy::highlight(pattern, "IGMP".to_string()).cyan(),
+                                    pid,
+                                ]),
                             },
                             IpPacket::V6(ipv6_packet) => match ipv6_packet.proto {
                                 IpProto::Tcp(p) => Row::new(vec![
@@ -433,14 +447,17 @@ impl Inspection {
                                     fuzzy::highlight(pattern, "ICMPv6".to_string()).cyan(),
                                     pid,
                                 ]),
+                                IpProto::Igmp(_) => {
+                                    // IGMP is for Ipv4 only
+                                    unreachable!()
+                                }
                             },
                         },
                     }
                 })
                 .collect()
         } else {
-            packets_to_display
-                .iter()
+            pdb.iter()
                 .map(|app_packet| {
                     let pid = match app_packet.pid {
                         Some(pid) => Span::from(pid.to_string()).into_centered_line().cyan(),
@@ -522,6 +539,18 @@ impl Inspection {
                                     Span::from("ICMPv4".to_string()).into_centered_line().cyan(),
                                     pid,
                                 ]),
+                                IpProto::Igmp(_) => Row::new(vec![
+                                    Span::from(ipv4_packet.src_ip.to_string())
+                                        .into_centered_line()
+                                        .blue(),
+                                    Span::from("-").into_centered_line().yellow(),
+                                    Span::from(ipv4_packet.dst_ip.to_string())
+                                        .into_centered_line()
+                                        .blue(),
+                                    Span::from("-").into_centered_line().yellow(),
+                                    Span::from("IGMP".to_string()).into_centered_line().cyan(),
+                                    pid,
+                                ]),
                             },
                             IpPacket::V6(ipv6_packet) => match ipv6_packet.proto {
                                 IpProto::Tcp(p) => Row::new(vec![
@@ -584,6 +613,10 @@ impl Inspection {
                                     Span::from("ICMPv6".to_string()).into_centered_line().cyan(),
                                     pid,
                                 ]),
+                                IpProto::Igmp(_) => {
+                                    // IGMP is for IPv4 only
+                                    unreachable!()
+                                }
                             },
                         },
                     }
@@ -592,11 +625,11 @@ impl Inspection {
         };
 
         // Always select the last packet
-        if !self.manuall_scroll {
+        if !self.manual_scroll {
             if fuzzy.is_enabled() {
-                fuzzy.scroll_state.select(Some(packets_to_display.len()));
+                fuzzy.scroll_state.select(Some(pdb.len()));
             } else {
-                self.state.select(Some(packets_to_display.len()));
+                self.state.select(Some(pdb.len()));
             }
         }
 
@@ -610,7 +643,7 @@ impl Inspection {
                     Line::from("Protocol").centered(),
                     Line::from("Pid").centered(),
                     {
-                        if self.manuall_scroll {
+                        if self.manual_scroll {
                             Line::from("󰹆").centered().yellow()
                         } else {
                             Line::from("").centered()
@@ -638,9 +671,10 @@ impl Inspection {
             .begin_symbol(Some("↑"))
             .end_symbol(Some("↓"));
 
+        let app_packets_len = self.packets.len();
         let mut scrollbar_state = if fuzzy.is_enabled() && fuzzy_packets.len() > window_size {
             ScrollbarState::new(fuzzy_packets.len()).position({
-                if self.manuall_scroll {
+                if self.manual_scroll {
                     if fuzzy.packet_end_index == window_size {
                         0
                     } else {
@@ -650,16 +684,16 @@ impl Inspection {
                     fuzzy.packets.len()
                 }
             })
-        } else if !fuzzy.is_enabled() && app_packets.len() > window_size {
-            ScrollbarState::new(app_packets.len()).position({
-                if self.manuall_scroll {
+        } else if !fuzzy.is_enabled() && app_packets_len > window_size {
+            ScrollbarState::new(app_packets_len).position({
+                if self.manual_scroll {
                     if self.packet_end_index == window_size {
                         0
                     } else {
                         self.packet_end_index
                     }
                 } else {
-                    app_packets.len()
+                    app_packets_len
                 }
             })
         } else {
@@ -733,12 +767,11 @@ impl Inspection {
             .split(layout[1])[1];
 
         let fuzzy = self.fuzzy.lock().unwrap();
-        let packets = self.packets.read().unwrap();
 
         let app_packet = if fuzzy.is_enabled() {
             fuzzy.packets[self.packet_index.unwrap()]
         } else {
-            packets[self.packet_index.unwrap()]
+            self.packets.get(self.packet_index.unwrap()).unwrap()
         };
 
         frame.render_widget(Clear, block);
