@@ -1,7 +1,7 @@
 use std::{
     cmp,
     ops::Range,
-    sync::{Arc, Mutex, RwLock, atomic::AtomicBool},
+    sync::{Arc, Mutex, atomic::AtomicBool},
     thread,
     time::Duration,
 };
@@ -24,10 +24,11 @@ use ratatui::{
 use crate::{
     app::AppResult,
     packet::{
-        AppPacket, NetworkPacket,
+        NetworkPacket,
         direction::TrafficDirection,
         network::{IpPacket, ip::IpProto},
     },
+    packet_store::PacketStore,
 };
 
 #[derive(Debug, Default)]
@@ -39,7 +40,7 @@ struct ListState {
 #[derive(Debug)]
 pub struct Metrics {
     user_input: UserInput,
-    app_packets: Arc<RwLock<Vec<AppPacket>>>,
+    app_packets: PacketStore,
     metrics: Vec<Arc<Mutex<PortCountMetric>>>,
     terminate: Arc<AtomicBool>,
     state: ListState,
@@ -96,7 +97,7 @@ pub struct PortCountMetric {
 }
 
 impl Metrics {
-    pub fn new(packets: Arc<RwLock<Vec<AppPacket>>>) -> Self {
+    pub fn new(packets: PacketStore) -> Self {
         Self {
             user_input: UserInput::default(),
             app_packets: packets,
@@ -168,13 +169,13 @@ impl Metrics {
                 .data(
                     BarGroup::default().bars(&[
                         Bar::default()
-                            .label("TCP".into())
+                            .label("TCP")
                             .style(Style::new().fg(Color::LightYellow))
                             .value(metric.tcp_count.try_into().unwrap())
                             .value_style(Style::new().fg(Color::Black).bg(Color::LightYellow))
                             .text_value(metric.tcp_count.to_string()),
                         Bar::default()
-                            .label("UDP".into())
+                            .label("UDP")
                             .style(Style::new().fg(Color::LightBlue))
                             .value_style(Style::new().fg(Color::Black).bg(Color::LightBlue))
                             .value(metric.udp_count.try_into().unwrap())
@@ -323,57 +324,55 @@ impl Metrics {
                     let packets = self.app_packets.clone();
                     move || {
                         let mut last_index = 0;
-                        'main: loop {
+                        loop {
                             thread::sleep(Duration::from_millis(100));
 
-                            let app_packets = { packets.read().unwrap().clone() };
-
-                            if app_packets.is_empty() {
+                            if packets.is_empty() {
                                 continue;
                             }
-                            let mut metric = port_count_metric.lock().unwrap();
-                            for app_packet in app_packets[last_index..].iter() {
-                                if terminate.load(std::sync::atomic::Ordering::Relaxed) {
-                                    break 'main;
-                                }
-                                if app_packet.direction == TrafficDirection::Ingress
-                                    && let NetworkPacket::Ip(packet) = app_packet.frame.payload
-                                {
-                                    match packet {
-                                        IpPacket::V4(ipv4_packet) => match ipv4_packet.proto {
-                                            IpProto::Tcp(tcp_packet) => {
-                                                if port_range.contains(&tcp_packet.dst_port) {
-                                                    metric.tcp_count += 1;
-                                                }
-                                            }
-                                            IpProto::Udp(udp_packet) => {
-                                                if port_range.contains(&udp_packet.dst_port) {
-                                                    metric.udp_count += 1;
-                                                }
-                                            }
-                                            _ => {}
-                                        },
-                                        IpPacket::V6(ipv6_packet) => match ipv6_packet.proto {
-                                            IpProto::Tcp(tcp_packet) => {
-                                                if port_range.contains(&tcp_packet.dst_port) {
-                                                    metric.tcp_count += 1;
-                                                }
-                                            }
-                                            IpProto::Udp(udp_packet) => {
-                                                if port_range.contains(&udp_packet.dst_port) {
-                                                    metric.udp_count += 1;
-                                                }
-                                            }
-                                            _ => {}
-                                        },
-                                    }
-                                }
-                            }
 
-                            last_index = app_packets.len();
+                            let mut metric = port_count_metric.lock().unwrap();
+
+                            last_index += packets
+                                .for_each_range(last_index.., |app_packet| {
+                                    if app_packet.direction == TrafficDirection::Ingress
+                                        && let NetworkPacket::Ip(packet) = app_packet.frame.payload
+                                    {
+                                        match packet {
+                                            IpPacket::V4(ipv4_packet) => match ipv4_packet.proto {
+                                                IpProto::Tcp(tcp_packet) => {
+                                                    if port_range.contains(&tcp_packet.dst_port) {
+                                                        metric.tcp_count += 1;
+                                                    }
+                                                }
+                                                IpProto::Udp(udp_packet) => {
+                                                    if port_range.contains(&udp_packet.dst_port) {
+                                                        metric.udp_count += 1;
+                                                    }
+                                                }
+                                                _ => {}
+                                            },
+                                            IpPacket::V6(ipv6_packet) => match ipv6_packet.proto {
+                                                IpProto::Tcp(tcp_packet) => {
+                                                    if port_range.contains(&tcp_packet.dst_port) {
+                                                        metric.tcp_count += 1;
+                                                    }
+                                                }
+                                                IpProto::Udp(udp_packet) => {
+                                                    if port_range.contains(&udp_packet.dst_port) {
+                                                        metric.udp_count += 1;
+                                                    }
+                                                }
+                                                _ => {}
+                                            },
+                                        }
+                                    }
+                                    Ok(())
+                                })
+                                .unwrap();
 
                             if terminate.load(std::sync::atomic::Ordering::Relaxed) {
-                                break 'main;
+                                break;
                             }
                         }
                     }
@@ -476,6 +475,16 @@ impl Metrics {
                 vertical: 0,
             }),
         );
+
+        let content_width = self.user_input.input.visual_cursor() as u16;
+        let mut cursor_x = form_block.x + block.width / 2 + 1 + content_width;
+        if !block.width.is_multiple_of(2) {
+            cursor_x += 1;
+        }
+
+        let cursor_y = form_block.y + 2;
+        frame.set_cursor_position((cursor_x, cursor_y));
+
         frame.render_widget(
             help_message,
             message_block.inner(Margin {
