@@ -21,7 +21,11 @@ impl<B: Backend> Tui<B> {
         Self { terminal, events }
     }
 
-    pub fn init(&mut self) -> AppResult<()> {
+    pub fn init(&mut self) -> AppResult<()>
+    where
+        <B as Backend>::Error: Sync + Send,
+        <B as Backend>::Error: 'static,
+    {
         terminal::enable_raw_mode()?;
         crossterm::execute!(io::stdout(), EnterAlternateScreen, EnableMouseCapture)?;
 
@@ -36,7 +40,11 @@ impl<B: Backend> Tui<B> {
         Ok(())
     }
 
-    pub fn draw(&mut self, app: &mut App) -> AppResult<()> {
+    pub fn draw(&mut self, app: &mut App) -> AppResult<()>
+    where
+        <B as Backend>::Error: Sync + Send,
+        <B as Backend>::Error: 'static,
+    {
         self.terminal.draw(|frame| ui::render(app, frame))?;
         Ok(())
     }
@@ -47,7 +55,11 @@ impl<B: Backend> Tui<B> {
         Ok(())
     }
 
-    pub fn exit(&mut self) -> AppResult<()> {
+    pub fn exit(&mut self) -> AppResult<()>
+    where
+        <B as Backend>::Error: Sync + Send,
+        <B as Backend>::Error: 'static,
+    {
         Self::reset()?;
         self.terminal.show_cursor()?;
         Ok(())
